#ifndef __LCPERLINNOISE2D_H__
#define __LCPERLINNOISE2D_H__


typedef float FLP32;
typedef unsigned int SYSUINT;

/** Usage:
 * First of all you should initialize all values (the standard values won't satisfy you :-) )
 * For example:
 *  SetPersistence( 0.6f );
 *  SetSeed( 12345 );
 *  SetDepth( 4 );
 * The amplitude depends on your application. For RGB values you usually require:
 *  SetAmplitude( 0.0f, 255.0f );
 * After you've called Create() you can start producing values on demand. Make sure you
 * call the GetValue function with an interval of about 0.01 depending on your application. If
 * you use an interval of >= 1 you won't notice any smoothing!
 */
class PerlinNoise2D
{

protected:
    SYSUINT     m_Seed;
    SYSUINT     m_Depth;
    FLP32       m_Amplitude;
    FLP32       m_Add;
    FLP32       m_Persistence;

    FLP32 Interpolate( FLP32 V1, FLP32 V2, FLP32 Step);
    FLP32 InterpolatedRand( FLP32 X, FLP32 Y);
    FLP32 SmoothRand( SYSUINT X, SYSUINT Y);
    FLP32 Rand( SYSUINT X, SYSUINT Y);
    
public:
    PerlinNoise2D();

    /** Set the minimal and maximal values for the result */
    void SetAmplitude( FLP32 Min, FLP32 Max);

    /** Set the persistence of the perlin noise */
    void SetPersistence( FLP32 Persistence);

    /** Set the random generator seed for the perlin noise
     * This is not the common random generator seed because this class has it's own
     * random generator. Seed can have any value.
     */
    void SetSeed( SYSUINT Seed);

    /** Set the calculation depth for the perlin noise */
    void SetDepth( SYSUINT Depth);

    /** Calculate additional values from the set parameters
     * This function has to be called for applying some of the Set*-functions.
     */
    void Create();

    /** Get a value for a given X and Y coordinate */
    FLP32 GetValue( FLP32 X, FLP32 Y);
    
};

#endif // __LCPERLINNOISE2D_H__
